import unittest

from tests.test_users import TestUsers
from tests.test_messaning import TestMessaging
from tests.test_peers import TestPeers
from tests.test_groups import TestGroups
from tests.test_updates import TestUpdates
from tests.test_manager import TestManager
from tests.test_interactive import TestInteractive
from tests.test_uploading import TestUploading
from tests.test_bot import TestBot
from tests.test_get_media import TestGetMedia

if __name__ == '__main__':
    unittest.main()
